<?php

namespace Drupal\clean_node_api\Controller\Helper;

use Drupal\file\Entity\File;
use Drupal\node\Entity\Node;
use Drupal\paragraphs\Entity\Paragraph;
use Drupal\taxonomy\Entity\Term;
use Drupal\user\Entity\User;

/**
 * Process fields.
 */
class Fields {

  public const FIELD = 'field_';

  public const EMPTY = '';

  public const VALUE = 'value';

  public const TARGET_ID = 'target_id';

  /**
   * Params.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param array $values
   *   Values Param.
   * @param string $key
   *   Key Param.
   *
   * @return array
   *   Return in array
   */
  public function booleanFields(string $fieldType, array $data, array $values, string $key): array {
    $newValues = NULL;
    if ($fieldType === 'boolean') {
      foreach ($values as $value) {
        $newValues = [self::VALUE => (bool) $value];
      }
      $data = $this->getArrayPush($data, $key, $newValues);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Params.
   *
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function getArrayPush(array $data, string $key, $values) {
    array_push($data, [str_replace(self::FIELD, self::EMPTY, $key) => $values]);
    // Return Data in array.
    return $data;
  }

  /**
   * Params.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param array $values
   *   Values Param.
   * @param string $key
   *   Key Param.
   *
   * @return array
   *   Return in array
   */
  public function changedFields(string $fieldType, array $data, array $values, string $key): array {

    if ($fieldType === 'changed') {
      $data = $this->changeTypeToInt($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Params.
   *
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function changeTypeToInt(array $data, string $key, array $values) {
    $newValues = NULL;
    foreach ($values as $newKey => $newValue) {
      $newValues[] = [self::VALUE => (int) $newValue[self::VALUE]];
    }
    return $this->getArrayPush($data, $key, $newValues);
  }

  /**
   * Params.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param array $values
   *   Values Param.
   * @param string $key
   *   Key Param.
   *
   * @return array
   *   Return in array
   */
  public function commentFields(string $fieldType, array $data, array $values, string $key): array {
    if ($fieldType === 'comment') {

      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Params.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param array $values
   *   Values Param.
   * @param string $key
   *   Key Param.
   *
   * @return array
   *   Return in array
   */
  public function createdFields(string $fieldType, array $data, array $values, string $key): array {

    if ($fieldType === 'created') {
      $data = $this->changeTypeToInt($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Params.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param array $values
   *   Values Param.
   * @param string $key
   *   Key Param.
   *
   * @return array
   *   Return in array
   */
  public function datetimeFields(string $fieldType, array $data, array $values, string $key): array {
    if ($fieldType === 'datetime') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Params.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param array $values
   *   Values Param.
   * @param string $key
   *   Key Param.
   *
   * @return array
   *   Return in array
   */
  public function daterangeFields(string $fieldType, array $data, array $values, string $key): array {
    if ($fieldType === 'daterange') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Params.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param array $values
   *   Values Param.
   * @param string $key
   *   Key Param.
   *
   * @return array
   *   Return in array
   */
  public function decimalFields(string $fieldType, array $data, array $values, string $key): array {
    if ($fieldType === 'decimal') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Params.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function emailFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'email') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Params.
   *
   * @param string $node
   *   Node Param.
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function entityReferenceFields($node, string $fieldType, array $data, string $key, array $values) {
    $entityData = NULL;
    if ($fieldType === 'entity_reference') {

      $entityData = $this->referenceControl($node, $key, $values, $entityData);

      $data = $this->getArrayPush($data, $key, $entityData);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Params.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function fileFields(string $fieldType, array $data, string $key, array $values) {
    $entityData = NULL;
    if ($fieldType === 'file') {
      foreach ($values as $id) {
        $entity = File::load($id[self::TARGET_ID]);
        if ($entity) {
          $entityData[] = [
            'fid' => (int) $entity->id(),
            'filename' => $entity->filename->value,
            'uri' => file_url_transform_relative(file_create_url($entity->getFileUri())),
          ];
        }
      }

      $data = $this->getArrayPush($data, $key, $entityData);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Params.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function floatFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'float') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Params.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function imageFields(string $fieldType, array $data, string $key, array $values) {
    $entityData = NULL;
    if ($fieldType === 'image') {
      foreach ($values as $id) {
        $entity = File::load($id[self::TARGET_ID]);
        if ($entity) {
          $entityData[] = [
            'fid' => (int) $entity->id(),
            'alt' => $id['alt'],
            'title' => $id['title'],
            'width' => (int) $id['width'],
            'height' => (int) $id['height'],
            'filename' => $entity->filename->value,
            'uri' => file_url_transform_relative(file_create_url($entity->getFileUri())),
          ];
        }
      }
      $data = $this->getArrayPush($data, $key, $entityData);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Params.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function integerFields(string $fieldType, array $data, string $key, array $values) {

    if ($fieldType === 'integer') {
      $data = $this->changeTypeToInt($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Params.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function languageFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'language') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Params.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function linkFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'link') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function listFloatFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'list_float') {

      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function listIntegerFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'list_integer') {
      $data = $this->changeTypeToInt($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function listStringFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'list_string') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function mapFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'map') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function passwordFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'password') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function pathFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'path') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function stringFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'string') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function stringLongFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'string_long') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function telephoneFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'telephone') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function textFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'text') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function textLongFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'text_long') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function textWithSummaryFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'text_with_summary') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function timestampFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'timestamp') {
      $data = $this->changeTypeToInt($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function uriFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'uri') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function uuidFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'uuid') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function dynamicEntityReferenceFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'dynamic_entity_reference') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $node
   *   Node param.
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function entityReferenceRevisionsFields($node, string $fieldType, array $data, string $key, array $values) {
    $entity = NULL;
    $entityData = NULL;
    if ($fieldType === 'entity_reference_revisions') {
      $entityType = $node->get($key)
        ->getFieldDefinition()
        ->getSettings()['target_type'];
      if ($entityType === 'paragraph') {
        foreach ($values as $id) {
          $entity = Paragraph::load($id[self::TARGET_ID]);
          if ($entity) {
            $entityData[] = [
              'pid' => (int) $entity->id(),
              'pData' => $this->rebuildFields($entity),
            ];
          }
        }
      }
      $data = $this->getArrayPush($data, $key, $entityData);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function videoFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'video') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function jqueryColorPickerFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'jquery_colorpicker') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Param.
   *
   * @param string $fieldType
   *   FieldType Param.
   * @param array $data
   *   Data Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Values Param.
   *
   * @return mixed
   *   Return in mixed .
   */
  public function metatagFields(string $fieldType, array $data, string $key, array $values) {
    if ($fieldType === 'metatag') {
      $data = $this->getArrayPush($data, $key, $values);
    }
    // Return Data in array.
    return $data;
  }

  /**
   * Reference Control .
   *
   * @param mixed $node
   *   Node Param.
   * @param string $key
   *   Key Param.
   * @param array $values
   *   Value Param.
   * @param array|null $entityData
   *   Entity Param.
   *
   * @return array|null
   *   Return array|null
   */
  public function referenceControl($node, string $key, array $values, ?array $entityData) {
    $entityType = $node->get($key)->getFieldDefinition()->getSettings()['target_type'];

    if ($entityType === 'user') {

      $entityData = $this->renderUserReference($values, $entityData);

    }
    elseif ($entityType === 'taxonomy_term') {

      $entityData = $this->renderTaxonomyTermReference($values, $entityData);

    }
    elseif ($entityType === 'node') {

      $entityData = $this->renderNodeReference($values, $entityData);

    }
    else {

      $entityData = $values;

    }
    // Return data.
    return $entityData;

  }

  /**
   * Render User Reference.
   *
   * @param array $values
   *   Value Param.
   * @param array|null $entityData
   *   EntityData Param.
   *
   * @return array|null
   *   Return array|null
   */
  public function renderUserReference(array $values, ?array $entityData) {
    foreach ($values as $id) {
      $entity = User::load($id[self::TARGET_ID]);
      if ($entity) {
        $entityData[] = [
          'uid' => (int) $entity->id(),
          'uuid' => $entity->uuid(),
          'name' => $entity->name->value,
        ];
      }
    }
    // Return Data.
    return $entityData;
  }

  /**
   * Render Taxonomy Term Reference.
   *
   * @param array $values
   *   Value Param.
   * @param array|null $entityData
   *   EntityData Param.
   *
   * @return array|null
   *   Return array|null
   */
  public function renderTaxonomyTermReference(array $values, ?array $entityData) {
    foreach ($values as $id) {
      $entity = Term::load($id[self::TARGET_ID]);
      if ($entity) {
        $entityData[] = [
          'tid' => (int) $entity->id(),
          'name' => $entity->name->value,
        ];
      }
    }
    // Return Data.
    return $entityData;
  }

  /**
   * Render Node Reference.
   *
   * @param array $values
   *   Value Param.
   * @param array|null $entityData
   *   EntityData Param.
   *
   * @return array|null
   *   Return array|null
   */
  public function renderNodeReference(array $values, ?array $entityData) {
    foreach ($values as $id) {
      $entity = Node::load($id[self::TARGET_ID]);
      if ($entity) {
        $entityData[] = [
          'nid' => (int) $entity->id(),
          'name' => $entity->getTitle(),
        ];
      }
    }
    // Return Data.
    return $entityData;
  }

}
