<?php

namespace Drupal\clean_node_api\Controller\Helper;

use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * MainHelper for Node class .
 */
class MainHelper extends Fields {

  /**
   * Params Description .
   *
   * @param mixed $body
   *   Return in $body.
   * @param string $message
   *   Return in $message.
   * @param int $statusCode
   *   Return in $statusCode.
   * @param string $statusMessage
   *   Return in $statusMessage.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The response information .
   */
  public static function returnDataInJson($body, $message, $statusCode, $statusMessage): JsonResponse {
    $data = [
      'body' => $body,
      'message' => $message,
    ];
    $jsonResponse = new JsonResponse($data);

    // Return value in json.
    return $jsonResponse->setStatusCode($statusCode, $statusMessage);
  }

  /**
   * Rebuild Fields.
   *
   * @param mixed $node
   *
   *   Return data in array .
   *
   * @return array
   *   Return data in array .
   */
  public function rebuildFields($node): array {
    $data = [];
    $array = $node->toArray();

    foreach ($array as $key => $values) {
      $fieldType = $node->get($key)->getFieldDefinition()->getType();

      $data = $this->booleanFields($fieldType, $data, $values, $key);

      $data = $this->changedFields($fieldType, $data, $values, $key);

      $data = $this->commentFields($fieldType, $data, $values, $key);

      $data = $this->createdFields($fieldType, $data, $values, $key);

      $data = $this->datetimeFields($fieldType, $data, $values, $key);

      $data = $this->daterangeFields($fieldType, $data, $values, $key);

      $data = $this->decimalFields($fieldType, $data, $values, $key);

      $data = $this->emailFields($fieldType, $data, $key, $values);

      $data = $this->entityReferenceFields($node, $fieldType, $data, $key, $values);

      $data = $this->fileFields($fieldType, $data, $key, $values);

      $data = $this->floatFields($fieldType, $data, $key, $values);

      $data = $this->imageFields($fieldType, $data, $key, $values);

      $data = $this->integerFields($fieldType, $data, $key, $values);

      $data = $this->languageFields($fieldType, $data, $key, $values);

      $data = $this->linkFields($fieldType, $data, $key, $values);

      $data = $this->listFloatFields($fieldType, $data, $key, $values);

      $data = $this->listIntegerFields($fieldType, $data, $key, $values);

      $data = $this->listStringFields($fieldType, $data, $key, $values);

      $data = $this->mapFields($fieldType, $data, $key, $values);

      $data = $this->passwordFields($fieldType, $data, $key, $values);

      $data = $this->pathFields($fieldType, $data, $key, $values);

      $data = $this->stringFields($fieldType, $data, $key, $values);

      $data = $this->stringLongFields($fieldType, $data, $key, $values);

      $data = $this->telephoneFields($fieldType, $data, $key, $values);

      $data = $this->textFields($fieldType, $data, $key, $values);

      $data = $this->textLongFields($fieldType, $data, $key, $values);

      $data = $this->textWithSummaryFields($fieldType, $data, $key, $values);

      $data = $this->timestampFields($fieldType, $data, $key, $values);

      $data = $this->uriFields($fieldType, $data, $key, $values);

      $data = $this->uuidFields($fieldType, $data, $key, $values);

      $data = $this->dynamicEntityReferenceFields($fieldType, $data, $key, $values);

      $data = $this->entityReferenceRevisionsFields($node, $fieldType, $data, $key, $values);

      $data = $this->videoFields($fieldType, $data, $key, $values);

      $data = $this->jqueryColorPickerFields($fieldType, $data, $key, $values);

      $data = $this->metatagFields($fieldType, $data, $key, $values);

    }

    // Push data in variable and return.
    return $data;
  }

}
